"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const git_1 = require('./git');
const migration_1 = require('./migration');
const git_2 = require('./git');
const chalk = require('chalk');
const fs = require('mz/fs');
const path = require('path');
const uuid = require('node-uuid');
const mkdirp = require('mkdirp');
const DEFAULT_LOGGER = {
    log: () => undefined,
    warn: () => undefined,
    error: () => undefined
};
class Status {
    /** Executes all tasks for newCommits */
    executePendingTasks(migrationDir, adapter, logger = DEFAULT_LOGGER) {
        return __awaiter(this, void 0, Promise, function* () {
            logger.log('Starting migration\n\n');
            for (const commit of this.newCommits) {
                logger.log(`${chalk.yellow(commit.shortSha1)} ${commit.subject}\n`);
                for (const task of commit.tasks) {
                    logger.log(task.toString() + '...');
                    const interval = setInterval(() => logger.log('.'), 100);
                    yield task.execute(migrationDir, adapter, this.head, commit);
                    clearInterval(interval);
                    logger.log(' Success\n');
                }
            }
            logger.log(chalk.green('\nAll migrations successful\n'));
        });
    }
    /** Returns a string that can be printed to a CLI */
    toString() {
        let str;
        if (this.lastTask) {
            str += `Last migration:      ${this.lastTask.toString()}\n`;
            str += `Applied at:          ${this.lastTask.appliedAt}\n`;
            str += `Triggered by commit: ${this.lastTask.commit.toString()}\n`;
            str += `HEAD at execution:   ${this.lastTask.commit.toString()}\n`;
        }
        else {
            str += `Last migration:      No migration run yet\n`;
        }
        if (this.head) {
            str += chalk.grey(`                        ${this.newCommits.length === 1 ? '‖' : `${this.newCommits.isReversed ? '↑' : '↓'} ${this.newCommits.length - 1} commit${this.newCommits.length > 2 ? 's' : ''}\n`}`);
            str += `Current HEAD:        ${this.head.toString()}\n`;
        }
        str += '\n';
        const relevantCommits = this.newCommits.filter(commit => commit.tasks.length > 0);
        if (relevantCommits.length === 0) {
            str += 'No pending migrations\n';
            return;
        }
        const migrationCount = relevantCommits.reduce((prev, curr) => prev + curr.tasks.length, 0);
        str += chalk.underline(`${migrationCount} pending migration${migrationCount > 1 ? 's' : ''}:\n\n`);
        for (const commit of relevantCommits) {
            str += commit.toString() + '\n';
            for (const task of commit.tasks) {
                str += (this.newCommits.isReversed ? task.invert() : task).toString() + '\n';
            }
            str += `\n`;
        }
        return str;
    }
}
exports.Status = Status;
function getStatus(adapter, head, migrationDir) {
    return __awaiter(this, void 0, Promise, function* () {
        const status = new Status();
        status.lastTask = yield adapter.getLastMigrationTask();
        status.head = head;
        if (status.lastTask) {
            status.newCommits = yield git_1.getNewCommits(status.lastTask.head);
            // Load commit messages
            yield Promise.all([status.lastTask.commit, status.lastTask.head].map((commit) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield commit.loadSubject();
                }
                catch (err) {
                    /* istanbul ignore next */
                    if (err.code !== 128) {
                        throw err;
                    }
                }
            })));
        }
        else {
            status.newCommits = yield git_1.getNewCommits();
        }
        if (head) {
            yield head.loadSubject();
        }
        return status;
    });
}
exports.getStatus = getStatus;
function isMerkelRepository() {
    return __awaiter(this, void 0, Promise, function* () {
        try {
            yield fs.access('.merkelrc.json');
            return true;
        }
        catch (err) {
            return false;
        }
    });
}
exports.isMerkelRepository = isMerkelRepository;
function createMigrationDir(migrationDir) {
    return __awaiter(this, void 0, void 0, function* () {
        yield new Promise((resolve, reject) => mkdirp(migrationDir, (err, made) => err ? reject(err) : resolve(made)));
    });
}
exports.createMigrationDir = createMigrationDir;
function createConfig(config) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs.writeFile('.merkelrc.json', JSON.stringify(config, null, 2) + '\n');
    });
}
exports.createConfig = createConfig;
function migrate(type, migrationDir, adapter, migrations, logger = DEFAULT_LOGGER) {
    return __awaiter(this, void 0, void 0, function* () {
        const head = yield git_2.getHead();
        for (const name of migrations) {
            const task = new migration_1.Task({
                type: 'up',
                migration: new migration_1.Migration({ name: name })
            });
            logger.log(`Executing ${task.toString()}...`);
            yield task.execute(migrationDir, adapter, head);
            logger.log(' Success\n');
        }
        logger.log('\n' + chalk.green.bold('Migration successful') + '\n');
    });
}
exports.migrate = migrate;
function prepareCommitMsg(msgfile, migrationDir, logger = DEFAULT_LOGGER) {
    return __awaiter(this, void 0, void 0, function* () {
        let msg = yield fs.readFile(msgfile, 'utf8');
        // check that migrations have not been deleted by a revert
        if (git_2.isRevertCommit(msg)) {
            logger.warn(chalk.bgYellow('WARNING: migrations have been removed by a git revert'));
        }
        // add commands
        const taskList = yield git_2.getTasksForNewCommit(migrationDir);
        yield fs.appendFile(msgfile, taskList.toString(true));
    });
}
exports.prepareCommitMsg = prepareCommitMsg;
function generate(options, logger = DEFAULT_LOGGER) {
    return __awaiter(this, void 0, void 0, function* () {
        options.name = options.name || uuid.v1();
        let template;
        let ext = '';
        if (options.template) {
            try {
                template = yield fs.readFile(options.template, 'utf8');
            }
            catch (err) {
                if (err.code !== 'ENOENT') {
                    throw err;
                }
                logger.error(chalk.red('\nCould not find template ' + options.template + '\n'));
            }
        }
        else {
            // detect tsconfig.json
            try {
                const tsconfig = JSON.parse(yield fs.readFile('tsconfig.json', 'utf8'));
                const targetLessThanEs6 = tsconfig.compilerOptions && /^es[35]$/i.test(tsconfig.compilerOptions.target);
                ext = '.ts';
                template = [
                    '',
                    `export ${targetLessThanEs6 ? '' : 'async '}function up(): Promise<void> {`,
                    '',
                    '}',
                    '',
                    `export ${targetLessThanEs6 ? '' : 'async '}function down(): Promise<void> {`,
                    '',
                    '}',
                    ''
                ].join('\n');
            }
            catch (err) {
                if (err.code !== 'ENOENT') {
                    throw err;
                }
                ext = '.js';
                template = [
                    '',
                    'exports.up = function up() {',
                    '',
                    '};',
                    '',
                    'exports.down = function down() {',
                    '',
                    '};',
                    ''
                ].join('\n');
            }
        }
        const file = `${options.migrationDir}/${options.name}${ext}`;
        const relativePath = path.relative(process.cwd(), options.migrationDir);
        // check if already exists
        try {
            yield fs.access(file);
            logger.error(chalk.red('\nError: Migration file ' + relativePath + path.sep + chalk.bold(options.name) + ext + ' already exists\n'));
            throw new Error('Migration file already existed');
        }
        catch (err) {
        }
        yield new Promise((resolve, reject) => mkdirp(options.migrationDir, err => err ? reject(err) : resolve()));
        yield fs.writeFile(file, template);
        logger.log('\nCreated ' + chalk.cyan(relativePath + path.sep + options.name + ext) + '\n');
    });
}
exports.generate = generate;
//# sourceMappingURL=index.js.map